library SL;

uses
  ShareMem,
  Windows,
  Registry,
  SysUtils,
  StrUtils,
  Classes,
  Dialogs,
  T35,
  CPort;

{$R *.res}

type
  TPassKey = record
    Reg: ShortString;      // Reg Path to the Modem
    Port: Byte;            // COM Port #
  end;
  TBlackList = record
    Status: byte;          // 0-blocked, 1-delayed, 2-free
    Phone: ShortString;    // Phone Number
    Delay: ShortString;    // Delay Time
    Attempts: ShortString; // Number of Attempts
  end;
  TList = Array of TBlackList;
  TStatistics = record
    Status: byte;          // 0-bad, 1-strange, 2-good
    Key: ShortString;      // Phone Number
    Value: ShortString;    // Delay Time
    Hint: String;          // Number of Attempts
  end;
  TStat = Array of TStatistics;
  TCountry = record
    ExactName: ShortString;
    MyName: ShortString;
    Code: Byte;
    Flag: Byte;
    Diz: String;
    Reserved: String;
  end;
  TCountryList = Array of TCountry;
  TRegister = record
    Name: ShortString; // Ex: S-XXX
    Number: Integer; // Pure
    Value: Integer;
    MinValue: Integer;
    MaxValue: Integer;
    Useful: Boolean; // Optional, for icons
    Diz: String;
  end;
  TRegisters = Array of TRegister;
  TTransmit = record
    TX_Data: Byte;
    TX_Fax: Byte;
    TX_Voice: Byte;
    TX_V90: Byte;
    TX_V92_1: Boolean;
    TX_V92_2: Boolean;
    TX_V92_3: Boolean;
    TX_V92_4: Boolean;
    TX_FData: Boolean;
    TX_FFax: Boolean;
    TX_FVoice: Boolean;
    TX_FV90: Boolean;
    TX_FV92_1: Boolean;
    TX_FV92_2: Boolean;
    TX_FV92_3: Boolean;
    TX_FV92_4: Boolean;
  end;
  TSignal = record
    Name: ShortString;
    Cycles: Byte;
    Filter: Byte;
    MinON: Byte;
    MaxON: Byte;
    MinOFF: Byte;
    MaxOFF: Byte;
  end;
  TSignals = Array [0..3] of TSignal;

const
  Plugin_Model: ShortString = 'Smart Link 1.0';
  Plugin_Driver: ShortString = '3.20.04';
  //Plugin_Driver: ShortString = '3.04.00';
  Plugin_Producer: ShortString = 'Neo Blue Team';
  Plugin_Released: ShortString = '11.06.2003';

var
  //Reg: TRegistry;
  ComPort: TComPort;
  //Events: TComEvents;
  //Data: string;
  //WaitResponse: boolean = false;

//-----------------------------------------------------------------

function GetPluginInfo: ShortString; stdcall;
begin
  Result := Plugin_Model + ' (up to ' + Plugin_Driver + ')';
end;

function GetModemModel(Key: TPassKey): ShortString; stdcall;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg, False);
    Result := Reg.ReadString('Model');
  finally
    Reg.Free;
  end;
end;

function GetCOMPort(Key: TPassKey): ShortString; stdcall;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg, False);
    if Reg.ValueExists('AttachedTo') then Result := Reg.ReadString('AttachedTo') else Result := '';
  finally
    Reg.Free;
  end;
end;

function GetCurrentCountry(Key: TPassKey): ShortString; stdcall;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg+'\HostBasedModemData\Parameters\DPDriver\Country', False);
    //Result := Reg.ReadString('CountryName') + ' (' + IntToStr(Reg.ReadInteger('CountryCode')) + ')';
    Result := Reg.ReadString('CountryName');
    Reg.CloseKey;
  finally
    Reg.Free;
  end;
end;

function GetTransmit(Key: TPassKey; var Trans: TTransmit): Boolean; stdcall;
var
  Reg: TRegistry;
  Country: shortstring;
  Buffer: array of byte;
  BufferSize: integer;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg+'\HostBasedModemData\Parameters\DPDriver\Country\HomologationParameters', False);
    Country := GetCurrentCountry(Key);
    BufferSize := Reg.GetDataSize(Country);
    SetLength(Buffer, BufferSize);
    Reg.ReadBinaryData(Country, Buffer[0], BufferSize);
    Trans.TX_Data := Buffer[46];
    Trans.TX_Fax := Buffer[98];
    Trans.TX_Voice := Buffer[62];
    Reg.CloseKey;
    Reg.OpenKey(Key.Reg+'\HostBasedModemData\Parameters\DPDriver', False);
    if Reg.ValueExists('V90MinTxPR') then
    begin
      Trans.TX_V90 := Reg.ReadInteger('V90MinTxPR');
      Trans.TX_FV90 := True;
    end
    else Trans.TX_FV90 := False;
    if Reg.ValueExists('QuickConnectEnabled') then
    begin
      Trans.TX_FV92_1 := True;
      case Reg.ReadInteger('QuickConnectEnabled') of
        0: Trans.TX_V92_1 := False;
        1: Trans.TX_V92_1 := True;
      else Trans.TX_FV92_1 := False;
      end;
    end
    else Trans.TX_FV92_1 := False;
    if Reg.ValueExists('PcmUpstreamEnabled') then
    begin
      Trans.TX_FV92_2 := True;
      case Reg.ReadInteger('PcmUpstreamEnabled') of
        0: Trans.TX_V92_2 := False;
        1: Trans.TX_V92_2 := True;
      else Trans.TX_FV92_2 := False;
      end;
    end
    else Trans.TX_FV92_2 := False;
    if Reg.ValueExists('V44Config') then
    begin
      Trans.TX_FV92_3 := True;
      case Reg.ReadInteger('V44Config') of
        0: Trans.TX_V92_3 := False;
        1: Trans.TX_V92_3 := True;
      else Trans.TX_FV92_3 := False;
      end;
    end
    else Trans.TX_FV92_3 := False;
    if Reg.ValueExists('ModemOnHoldEnabled') then
    begin
      Trans.TX_FV92_4 := True;
      case Reg.ReadInteger('ModemOnHoldEnabled') of
        0: Trans.TX_V92_4 := False;
        1: Trans.TX_V92_4 := True;
      else Trans.TX_FV92_4 := False;
      end;
    end
    else Trans.TX_FV92_4 := False;
  finally
    Reg.Free;
  end;
end;

function SetTransmit(Key: TPassKey; var Trans: TTransmit): Boolean; stdcall;
var
  Reg: TRegistry;
  Country: shortstring;
  Buffer: array of byte;
  BufferSize: integer;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg+'\HostBasedModemData\Parameters\DPDriver\Country\HomologationParameters', False);
    Country := GetCurrentCountry(Key);
    BufferSize := Reg.GetDataSize(Country);
    SetLength(Buffer, BufferSize);
    Reg.ReadBinaryData(Country, Buffer[0], BufferSize);
    //if Trans.TX_FData then
    Buffer[46] := Trans.TX_Data;
    //if Trans.TX_FFax then
    Buffer[98] := Trans.TX_Fax;
    //if Trans.TX_FVoice then
    Buffer[62] := Trans.TX_Voice;
    Reg.WriteBinaryData(Country, Buffer[0], BufferSize);
    Reg.CloseKey;
    Reg.OpenKey(Key.Reg+'\HostBasedModemData\Parameters\DPDriver', False);
    if Trans.TX_FV90 then Reg.WriteInteger('V90MinTxPR',Trans.TX_V90);
    if Trans.TX_FV92_1 then (if Trans.TX_V92_1 then Reg.WriteInteger('QuickConnectEnabled',1) else Reg.WriteInteger('QuickConnectEnabled',0));
    if Trans.TX_FV92_2 then (if Trans.TX_V92_2 then Reg.WriteInteger('PcmUpstreamEnabled',1) else Reg.WriteInteger('PcmUpstreamEnabled',0));
    if Trans.TX_FV92_3 then (if Trans.TX_V92_3 then Reg.WriteInteger('V44Config',1) else Reg.WriteInteger('V44Config',0));
    if Trans.TX_FV92_4 then (if Trans.TX_V92_4 then Reg.WriteInteger('ModemOnHoldEnabled',1) else Reg.WriteInteger('ModemOnHoldEnabled',0));
  finally
    Reg.Free;
  end;
end;

function DualTiming(Key: TPassKey; var DT1, DT2, A1, A2, A1_min, A1_max: byte; Store: Boolean): Boolean; stdcall;
var
  Reg: TRegistry;
  Country: shortstring;
  Buffer: array of byte;
  BufferSize: integer;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg+'\HostBasedModemData\Parameters\DPDriver\Country\HomologationParameters', False);
    Country := GetCurrentCountry(Key);
    BufferSize := Reg.GetDataSize(Country);
    SetLength(Buffer, BufferSize);
    Reg.ReadBinaryData(Country, Buffer[0], BufferSize);
    if Store then
    begin
      Buffer[86] := DT1;
      Buffer[64] := DT2;
      Buffer[92] := A1;
      Buffer[87] := A2;
      Reg.WriteBinaryData(Country, Buffer[0], BufferSize);
    end
    else
    begin
      DT1 := Buffer[86];
      DT2 := Buffer[64];
      A1 := Buffer[92];
      A1_min := Buffer[40];
      A1_max := Buffer[41];
      A2 := Buffer[87];
    end;
    Reg.CloseKey;
  finally
    Reg.Free;
  end;
end;

procedure DualSignals(Key: TPassKey; var Signals: TSignals; Store: Boolean); stdcall;
var
  Reg: TRegistry;
  Country: shortstring;
  Buffer: array of byte;
  BufferSize: integer;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg+'\HostBasedModemData\Parameters\DPDriver\Country\HomologationParameters', False);
    Country := GetCurrentCountry(Key);
    BufferSize := Reg.GetDataSize(Country);
    SetLength(Buffer, BufferSize);
    Reg.ReadBinaryData(Country, Buffer[0], BufferSize);
    if Store then
    begin
      //Signals[0].Name := 'Busy';
      Buffer[16] := Signals[0].MinON;
      Buffer[17] := Signals[0].MaxON;
      Buffer[18] := Signals[0].Cycles;
      Buffer[19] := Signals[0].MinOFF;
      Buffer[20] := Signals[0].MaxOFF;
      Buffer[83] := Signals[0].Filter;

      //Signals[1].Name := 'Congestion';
      Buffer[106] := Signals[1].MinON;
      Buffer[107] := Signals[1].MaxON;
      Buffer[108] := Signals[1].Cycles;
      Buffer[109] := Signals[1].MinOFF;
      Buffer[110] := Signals[1].MaxOFF;
      Buffer[111] := Signals[1].Filter;

      //Signals[2].Name := 'Ring Back';
      Buffer[100] := Signals[2].MinON;
      Buffer[101] := Signals[2].MaxON;
      Buffer[102] := Signals[2].Cycles;
      Buffer[103] := Signals[2].MinOFF;
      Buffer[114] := Signals[2].MaxOFF;
      Buffer[105] := Signals[2].Filter;

      Reg.WriteBinaryData(Country, Buffer[0], BufferSize);
    end
    else
    begin
      Signals[0].Name := 'Busy';
      Signals[0].MinON := Buffer[16];
      Signals[0].MaxON := Buffer[17];
      Signals[0].Cycles := Buffer[18];
      Signals[0].MinOFF := Buffer[19];
      Signals[0].MaxOFF := Buffer[20];
      Signals[0].Filter := Buffer[83];

      Signals[1].Name := 'Congestion';
      Signals[1].MinON := Buffer[106];
      Signals[1].MaxON := Buffer[107];
      Signals[1].Cycles := Buffer[108];
      Signals[1].MinOFF := Buffer[109];
      Signals[1].MaxOFF := Buffer[110];
      Signals[1].Filter := Buffer[111];

      Signals[2].Name := 'Ring Back';
      Signals[2].MinON := Buffer[100];
      Signals[2].MaxON := Buffer[101];
      Signals[2].Cycles := Buffer[102];
      Signals[2].MinOFF := Buffer[103];
      Signals[2].MaxOFF := Buffer[114];
      Signals[2].Filter := Buffer[105];
    end;
  finally
    Reg.Free;
  end;
end;

function GetInfoChipset(Key: TPassKey): ShortString; stdcall;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg, False);
    if Reg.ValueExists('Info.Chipset') then Result := Reg.ReadString('Info.Chipset') else Result := '';
  finally
    Reg.Free;
  end;
end;

function GetInfoMC(Key: TPassKey): ShortString; stdcall;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg, False);
    if Reg.ValueExists('Info.MC') then Result := Reg.ReadString('Info.MC') else Result := '';
  finally
    Reg.Free;
  end;
end;

function GetInfoAC(Key: TPassKey): ShortString; stdcall;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg, False);
    if Reg.ValueExists('Info.AC') then Result := Reg.ReadString('Info.AC') else Result := '';
  finally
    Reg.Free;
  end;
end;

function GetDriverVersion(Key: TPassKey): ShortString; stdcall;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg, False);
    if Reg.ValueExists('DriverVersion') then Result := Reg.ReadString('DriverVersion') else Result := '';
  finally
    Reg.Free;
  end;
end;

function DualUserInit(Key: TPassKey; var Init: ShortString; Store: Boolean): Boolean; stdcall;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  Result := False;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg, False);
    if Store then
    begin
      if (Init <> '') then Reg.WriteString('UserInit', Init) else Reg.DeleteValue('UserInit');
      Result := True;
    end;
    if Reg.ValueExists('UserInit') then
    begin
      Init := Reg.ReadString('UserInit');
      Result := True;
    end;
  finally
    Reg.Free;
  end;
end;

function GetHardwareID(Key: TPassKey): ShortString; stdcall;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg, False);
    Result := Reg.ReadString('MatchingDeviceId');
  finally
    Reg.Free;
  end;
end;

function GetOtherInfo(Key: TPassKey; var In1, In2, In3, In4: ShortString): Boolean; stdcall;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg, False);
    if Reg.ValueExists('SWVendor') then In1 := Reg.ReadString('SWVendor') else In1 := '';
    if Reg.ValueExists('SWVersion') then In2 := Reg.ReadString('SWVersion') else In2 := '';
    if Reg.ValueExists('HWVendor') then In3 := Reg.ReadString('HWVendor') else In3 := '';
    if Reg.ValueExists('HWVersion') then In4 := Reg.ReadString('HWVersion') else In4 := '';
  finally
    Reg.Free;
  end;
end;

function GetLoggingPath(Key: TPassKey; var Path: String): boolean ; stdcall;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  Result := false;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg, False);
    Path := Reg.ReadString('LoggingPath');
    if Path <> '' then Result := true;
  finally
    Reg.Free;
  end;
end;

function SendAT(Key: TPassKey; AT: ShortString): String; stdcall;
var
  Data: string;
begin
  try
    ComPort := TComPort.Create(nil);
    ComPort.Port := 'COM'+IntToStr(Key.Port);
    //ComPort.Events := [];
    ComPort.FlowControl.ControlDTR := dtrEnable;
    ComPort.FlowControl.ControlRTS := rtsEnable;
    ComPort.Open;
    ComPort.WriteStr(AT+#13#10);
    //ComPort.WaitForEvent(Events, 0, 2000);
    Sleep(500);
    ComPort.ReadStr(Data, ComPort.InputCount);
  finally
    ComPort.Free;
  end;
  Result := Data;
end;

function GetBlacklist(Key: TPassKey; var BL: TList): integer; stdcall;
const
  Digits: String = '0123456789';
var
  I, J: integer;
  Old, New: TStrings;
begin
  Old := TStringList.Create;
  New := TStringList.Create;
  try
    Old.Text := SendAT(Key, 'AT*B');
    for I := 0 to Old.Count-1 do
    begin
      if (Length(Old[I]) > 19) and (AnsiContainsStr(Digits, Old[I][1]))
      then New.Add(Old[I]);
    end;
    SetLength(BL, New.Count);
    for J := 0 to New.Count-1 do
    begin
      BL[J].Attempts := IntToStr(StrToInt(Trim(Copy(New[J],4,6))));
      if BL[J].Attempts[Length(BL[J].Attempts)] = '1' then
      BL[J].Attempts := BL[J].Attempts + ' time' else
      BL[J].Attempts := BL[J].Attempts + ' times';
      BL[J].Delay := Trim(Copy(New[J],10,8));
      BL[J].Phone := Trim(Copy(New[J],19,Length(New[J])-18));
      if not AnsiContainsStr(Digits, BL[J].Phone[1]) then
      BL[J].Phone := Copy(BL[J].Phone,2,Length(BL[J].Phone)-1);
      If BL[J].Delay = '' then BL[J].Status := 2
      else
      begin
        If BL[J].Delay = '*' then
        begin
          BL[J].Status := 0;
          BL[J].Delay := 'blocked';
        end
        else BL[J].Status := 1;
      end;
    end;
    Result := New.Count;
  finally
    begin
      Old.Free;
      New.Free;
    end;
  end;
end;

function DualBlacklistParams(Key: TPassKey; Store: boolean; var BL_Enabled: boolean; var Attempts, Timeout, Duration: byte): boolean; stdcall;
var
  Reg: TRegistry;
  Country: shortstring;
  Buffer: array of byte;
  BufferSize: integer;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg+'\HostBasedModemData\Parameters\DPDriver\Country\HomologationParameters', False);
    Country := GetCurrentCountry(Key);
    BufferSize := Reg.GetDataSize(Country);
    SetLength(Buffer, BufferSize);
    Reg.ReadBinaryData(Country, Buffer[0], BufferSize);
    if Store then
    begin
      if BL_Enabled then Buffer[23] := 1 else Buffer[23] := 0;
      Buffer[24] := Attempts;
      Buffer[93] := Timeout;
      Buffer[28] := Duration;
      Reg.WriteBinaryData(Country, Buffer[0], BufferSize);
    end
    else
    begin
      if Buffer[23] = 0 then BL_Enabled := false else BL_Enabled := true;
      Attempts := Buffer[24];
      Timeout := Buffer[93];
      Duration  := Buffer[28];
    end;
  finally
    Reg.Free;
  end;
end;

function GetCountries(Key: TPassKey; var Countries: TCountryList): integer; stdcall;
var
  Reg: TRegistry;
  Values: TStrings;
  I, J: Integer;
  Buffer: array of byte;
  BufferSize: integer;
begin
  Reg := TRegistry.Create;
  Values := TStringList.Create;
  Result := 0;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg+'\HostBasedModemData\Parameters\DPDriver\Country\HomologationParameters', False);
    Reg.GetValueNames(Values);
    SetLength(Countries, Values.Count);
    J := 0;
    for I := 0 to (Values.Count - 1) do
    begin
      BufferSize := Reg.GetDataSize(Values[I]);
      if (Reg.GetDataType(Values[I]) = rdBinary) and (BufferSize > 140) then
      begin
        SetLength(Buffer, BufferSize);
        Countries[J].ExactName := Values[I];
        Reg.ReadBinaryData(Values[I], Buffer[0], BufferSize);
        Countries[J].Code := Buffer[130];
        Countries[J].MyName := T35_CountryCode[Countries[J].Code];
        // Countries[J].MyName := T35_CountryCode[Countries[J].Code] + ' [' + Countries[J].ExactName + ']';
        if Countries[J].Code > 200 then Countries[J].Flag := 8
        else Countries[J].Flag := Countries[J].Code;
        // Mistakes of Smart Link
        if Countries[J].ExactName = 'ESTONIA' then
        begin
          Countries[J].MyName := 'Estonia';
          Countries[J].Flag := 213;
        end;
        if Countries[J].ExactName = 'LATVIA' then
        begin
          Countries[J].MyName := 'Latvia';
          Countries[J].Flag := 222;
        end;
        if Countries[J].ExactName = 'LITHUANIA' then
        begin
          Countries[J].MyName := 'Lithuania';
          Countries[J].Flag := 223;
        end;
        if Countries[J].ExactName = 'SLOVAKIA' then
        begin
          Countries[J].MyName := 'Slovakia';
          Countries[J].Flag := 197;
        end;
        if Countries[J].ExactName = 'SLOVENIA' then
        begin
          Countries[J].MyName := 'Slovenia';
          Countries[J].Flag := 198;
        end;
        if Countries[J].ExactName = 'TAIWAN' then
        begin
          Countries[J].MyName := 'Taiwan';
          Countries[J].Flag := 239;
        end;
        if Countries[J].ExactName = 'CTR21EUROPE' then
        begin
          Countries[J].MyName := 'Europe';
          Countries[J].Flag := 199;
        end;
        if Countries[J].ExactName = 'KOREA' then
        begin
          Countries[J].MyName := 'North Korea';
          Countries[J].Flag := 48;
        end;
        // Discription starts
        Countries[J].Diz := '';
        if Buffer[57] = 0 then Countries[J].Diz := Countries[J].Diz + 'Pulse dialing. '
        else if Buffer[57] = 1 then Countries[J].Diz := Countries[J].Diz + 'Tone dialing. '
        else if Buffer[57] = 2 then Countries[J].Diz := Countries[J].Diz + 'Pulse dialing disabled. ';
        if Buffer[130] = 1 then Countries[J].Diz := Countries[J].Diz + 'Caller ID detection. ';
        if Buffer[23] = 1 then Countries[J].Diz := Countries[J].Diz + 'Blacklisting. ';
        if Buffer[46] = 0 then Countries[J].Diz := Countries[J].Diz + 'Transmit level 0 dBm. '
        else Countries[J].Diz := Countries[J].Diz + 'Transmit level -' + IntToStr(Buffer[46]) + ' dBm. ';
        if Buffer[16] = 0 then Countries[J].Diz := Countries[J].Diz + 'Default BUSY signal parameters. '
        else Countries[J].Diz := Countries[J].Diz + 'BUSY detection in ' + IntToStr(Buffer[18]) + ' cycles, ON: ' + IntToStr(Buffer[16]*10) + ' - ' + IntToStr(Buffer[17]*10) + ' ms, OFF: ' + IntToStr(Buffer[19]*10) + ' - ' + IntToStr(Buffer[20]*10) + ' ms. ';
        Countries[J].Diz := Countries[J].Diz + 'Dial tone threshold -' + IntToStr(Buffer[59]) + ' dBm.';
        // Diz ends
        Inc(J);
      end;
    end;
    SetLength(Countries, J);
    Result := J;
  finally
    Values.Free;
    Reg.Free;
  end;
end;

function SetCurrentCountry(Key: TPassKey; Country: TCountry): Boolean; stdcall;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  Result := false;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg+'\HostBasedModemData\Parameters\DPDriver\Country', False);
    Reg.WriteString('CountryName', Country.ExactName);
    Reg.WriteInteger('ApplyNow', 1);
    Result := true;
    Reg.CloseKey;
  finally
    Reg.Free;
  end;
end;

procedure DualRegisters(Key: TPassKey; Store: boolean; var Regs: TRegisters; var RegsAmount: byte); stdcall;
var
  Reg: TRegistry;
  Values: TStrings;
  I: Integer;
begin
  Reg := TRegistry.Create;
  Values := TStringList.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(Key.Reg+'\SRegs', False);
    if Store then
    begin
      for I := 0 to RegsAmount do
      begin
        Reg.WriteString(IntToStr(Regs[I].Number), IntToStr(Regs[I].Value));
      end;
    end
    else
    begin
      Reg.GetValueNames(Values);
      RegsAmount := Values.Count - 1;
      SetLength(Regs, Values.Count);
      for I := 0 to RegsAmount do
      begin
        Regs[I].Name := 'S-' + Values[I];
        Regs[I].Number := StrToInt(Values[I]);
        Regs[I].Value := StrToInt(Reg.ReadString(Values[I]));
        Regs[I].MinValue := 0;
        Regs[I].MaxValue := 255;
      end;
    end;
  finally
    Values.Free;
    Reg.Free;
  end;
end;

//-----------------------------------------------------------------

exports
  GetPluginInfo,
  GetModemModel,
  GetHardwareID,
  GetLoggingPath,
  GetCurrentCountry,
  SendAT,
  GetBlacklist,
  DualBlacklistParams,
  GetCountries,
  SetCurrentCountry,
  DualRegisters,
  DualUserInit,
  GetCOMPort,
  GetDriverVersion,
  GetInfoChipset,
  GetInfoMC,
  GetOtherInfo,
  GetTransmit,
  GetInfoAC,
  DualSignals,
  DualTiming,
  SetTransmit;

begin
end.

