unit cCiaOnlyOne;

{
   Drop this component on the main form of your project. Check eventually the
   creation order that this code gets first executed.

   If you leave the Unique property empty, then it is possible to execute the
   program from out different directories (I needed this feature). If you put
   an unique string in it, then only one instance of the program can run.

   When an instance of the program is already running it will be set to
   foreground and this instance will stop, without showing a button or so.

   Any comments, sugestions are welcome at: wilfried@mestdagh.biz
   See also http://www.mestdagh.biz

   Version information:

   1.01 7 jan 2002
      - addes property AlreadyRunning. It can be used in the application if
        some cleanupcode is done, wich is normally not nececary if the program
        is already running.
}

interface

uses
  Windows, Classes, Forms;

type
  TCiaOnlyOne = class(TComponent)
  private
    FAlreadyRunning: boolean;
    hMutex: THandle;
    FUnique: string;
    hPrev: HWND;
    procedure CheckIfOnlyOne;
  protected
  public
    procedure  Loaded; override;
    destructor Destroy; override;
    property AlreadyRunning: boolean read FAlreadyRunning;
  published
    property Unique: string read FUnique write FUnique;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Cia', [TCiaOnlyOne]);
end;

procedure TCiaOnlyOne.Loaded;
begin
   inherited;
   if not (csDesigning in ComponentState) then
      CheckIfOnlyOne;
end;

procedure TCiaOnlyOne.CheckIfOnlyOne;
var
   Ca: string[255];
   n: integer;
begin
   if FUnique = '' then
      FUnique := Application.ExeName;
   for n := 1 to length(FUnique) do              // remove non permitted char's
      if FUnique[n] in ['\', ' '] then
         FUnique[n] := '_';

   hMutex := CreateMutex(nil, True, PChar(FUnique));
   if GetLastError = ERROR_ALREADY_EXISTS then
   try
      try
         Ca := Application.Title;
         Ca[ord(Ca[0]) + 1] := #0;
         Application.Title := '';               // dont let it find this window
         HPrev := FindWindow('TApplication', @Ca[1]);
         if IsIconic(hPrev) then
            ShowWindow(hPrev, SW_RESTORE);

         SetForeGroundWindow(HPrev);
         Application.ShowMainForm := False;     // dont show a button or form
      finally
         FAlreadyRunning := True;
         Application.Terminate;
      end;
   except
   end;
end;

destructor TCiaOnlyOne.Destroy;
begin
   try
      CloseHandle(hMutex);
   except
   end;
   inherited;
end;

end.
